import 'dart:mirrors';

class DependencyNotFoundException implements Exception {
  final Type dependencyType;

  DependencyNotFoundException(this.dependencyType);

  @override
  String toString() => 'DependencyNotFoundException: undeclared is $dependencyType';
}

class DependenciesContainer {
  final _cache = <Type, Object>{};
  final _factory = <Type, Object Function()>{};

  void update<T extends Object>(T Function(DependenciesContainer container) getter) {
    _factory[T] = () => getter(this);
  }

  U dependency<U>({bool existingInstance = true}) {
    final key = _factory.keys.firstWhere((element) => reflectClass(element).isSubtypeOf(reflectClass(U)));
    if (existingInstance) {
      if (_cache[key] == null) {
        _cache[key] = _factory[key]!();
      }
      return _cache[key]! as U;
    } else {
      return _factory[key]!() as U;
    }
  }

  void clearSingleton<U>() => _cache.remove(U);

  void clear() {
    _cache.clear();
    _factory.clear();
  }
}
